/**
 *
 * AbstractConverter.java
 *
 * Copyright 2012 Infoblox Inc. All rights reserved.
 *             
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *     2. Redistributions in binary form must reproduce the above copyright 
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY INFOBLOX AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL INFOBLOX OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * The views and conclusions contained in the software and documentation are
 * those of the authors and should not be interpreted as representing official
 * policies, either expressed or implied, of Infoblox.
 *
 */

package com.infoblox.wapi.converter;

import java.io.BufferedReader;
import java.io.DataInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.StringWriter;
import java.io.Writer;

import org.apache.log4j.Logger;
import org.w3c.dom.Document;

import com.sun.org.apache.xml.internal.serialize.OutputFormat;
import com.sun.org.apache.xml.internal.serialize.XMLSerializer;

/**
 * Abstract WAPI converter.
 * 
 * @param <T>
 */
public abstract class AbstractConverter<T> {
    /** Log4j logger */
    static private Logger logger = Logger.getLogger(AbstractConverter.class);

    /**
     * @param document
     * @return Formatted string with XML document.
     */
    public static String documentToString(Document document) {
        return documentToString(document, null);
    }

    /**
     * @param document
     * @param format
     *            OutputFormat for this document
     * @return Formatted string with XML document.
     */
    public static String documentToString(Document document, OutputFormat format) {
        OutputFormat outputFormat = format;
        if (outputFormat == null) {
            // Create basic format with magic numbers.
            outputFormat = new OutputFormat(document);
            outputFormat.setLineWidth(65);
            outputFormat.setIndenting(true);
            outputFormat.setIndent(2);
        }
        Writer out = new StringWriter();
        XMLSerializer serializer = new XMLSerializer(out, outputFormat);
        try {
            serializer.serialize(document);
        } catch (IOException e) {
            logger.error(
                    "Exception occurred while formatting document to string", e); //$NON-NLS-1$
            throw new RuntimeException(e);
        }
        return out.toString();
    }

    /**
     * @param stream
     * @return String from stream.
     */
    public static String streamToString(InputStream stream) {
        DataInputStream responseStream = new DataInputStream(stream);
        BufferedReader br = new BufferedReader(new InputStreamReader(
                responseStream));

        StringBuilder sb = new StringBuilder();
        String line;
        try {
            while ((line = br.readLine()) != null) {
                sb.append(line);
            }
            responseStream.close();
        } catch (IOException e) {
            logger.error(
                    "Exception occurred while getting string from Input stream", e); //$NON-NLS-1$
            throw new RuntimeException(e);
        }
        return sb.toString();
    }

    /**
     * @param string
     * @return Object created based on input string.
     */
    public abstract T stringToObject(String string);

    /**
     * @param object
     * @return String created based on input object.
     */
    public abstract String objectToString(T object);

}
