/**
 *
 * IWapiDAO.java
 *
 * Copyright 2012 Infoblox Inc. All rights reserved.
 *             
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *     2. Redistributions in binary form must reproduce the above copyright 
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY INFOBLOX AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL INFOBLOX OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * The views and conclusions contained in the software and documentation are
 * those of the authors and should not be interpreted as representing official
 * policies, either expressed or implied, of Infoblox.
 *
 */
package com.infoblox.wapi.dao;

import com.infoblox.wapi.converter.AbstractConverter;

/**
 * Interface for the WapiDAO.
 * 
 * @param <I>
 * @param <O>
 */
public interface IWapiDAO<I, O> {

    /**
     * @param ref
     * @return Object which was found by ref.
     */
    public O read(String ref);

    /**
     * Read object by ref in URL.
     * 
     * @return Object which was found by ref in URL which was set in the config.
     */
    public O read();

    /**
     * @return All objects with specified object type.
     */
    public O readAll();

    /**
     * @param object
     * @return String with ref of created object.
     */
    public String create(I object);

    /**
     * @param ref
     * @param object
     * @return String with ref of updated object.
     */
    public String update(String ref, I object);

    /**
     * Update object by ref in URL.
     * 
     * @param object
     * @return String with ref of updated object.
     */
    public String update(I object);

    /**
     * @param ref
     * @return String with ref of deleted object.
     */
    public String delete(String ref);

    /**
     * Delete object by ref in URL which was set in the config.
     * 
     * @return String with ref of deleted object.
     */
    public String delete();

    /**
     * @param object
     * @return Objects that satisfy the search query.
     */
    public O search(I object);

    /**
     * @return AbstractConverter for input objects.
     */
    public AbstractConverter<I> getInputConverter();

    /**
     * @return AbstractConverter for output objects.
     */
    public AbstractConverter<O> getOutputConverter();
}
